package eu.atosresearch.health.prescripciones.client;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import eu.atosresearch.health.prescripciones.client.widgets.AutorizacionWidget;
import eu.atosresearch.health.prescripciones.client.widgets.PrincipalAlarmasWidget;
import eu.atosresearch.health.prescripciones.client.widgets.UsuarioDelSistemaWidget;
import eu.atosresearch.health.prescripciones.shared.PacienteBean;


public class Controlador {
	
	private static Controlador INSTANCE = new Controlador();
	private final BackendServiceAsync backendService = GWT.create(BackendService.class);
	
	private final static int SEGUNDOSPOLLING = 30;
	
	private String usuarioSistema = " ";
	private List<PacienteBean> pacientes;
	
	private RootPanel rootPanel;
	private VerticalPanel mainVerticalRootPanel;
	
	private PrincipalAlarmasWidget menuWidget;


	//SINGLETON
	private Controlador() {
	}
 
	public static Controlador getInstance() {
        return INSTANCE;
    }    
	
	//GETTERS AND SETTERS

	public String getUsuarioSistema() {
		return usuarioSistema;
	}

	public void setUsuarioSistema(String usuarioSistema) {
		this.usuarioSistema = usuarioSistema;
	}

	public RootPanel getRootPanel() {
		return rootPanel;
	}

	public void setRootPanel(RootPanel rootPanel) {
		this.rootPanel = rootPanel;
	}

	public PrincipalAlarmasWidget getMenuWidget() {
		return menuWidget;
	}

	public void setMenuWidget(PrincipalAlarmasWidget menuWidget) {
		this.menuWidget = menuWidget;
	}
	
	public VerticalPanel getMainVerticalRootPanel() {
		return mainVerticalRootPanel;
	}

	public void setMainVerticalRootPanel(VerticalPanel mainVerticalRootPanel) {
		this.mainVerticalRootPanel = mainVerticalRootPanel;
	}
	
	public List<PacienteBean> getPacientes() {
		return pacientes;
	}

	public void setPacientes(List<PacienteBean> pacientes) {
		this.pacientes = pacientes;
	}
	
	
	//MÉTODOS
	

	//Cuando termina, hay nombreUsuarioSistema y pasa a cargarPautas
	public void autorizar() {
		Log.debug("Autorización desde el controlador");
		rootPanel.clear();
		AutorizacionWidget autorizacionWidget = new AutorizacionWidget();
		rootPanel.add(autorizacionWidget);
		Log.debug("Widget de autorización cargado");
		//Síncrono, redirige a cargarMenu
	}
	
	public void cabeceraUsuarioSistema(String usuario) {
		Log.debug("Creando cabecera USUARIO DEL SISTEMA para (" + usuario + ")");    	
    	UsuarioDelSistemaWidget usuarioDelSistema = new UsuarioDelSistemaWidget(usuario);
    	mainVerticalRootPanel.add(usuarioDelSistema);
    	Log.debug("Cabecera USUARIO DEL SISTEMA cargada (" + usuario + ")");
	}
	
	public void dejarSoloLaCabecera() {
		rootPanel.clear();
		ScrollPanel mainScrollPanel = new ScrollPanel();
		rootPanel.add(mainScrollPanel);
		mainVerticalRootPanel = new VerticalPanel();
		mainVerticalRootPanel.setWidth("100%");
		mainScrollPanel.add(mainVerticalRootPanel);
		cabeceraUsuarioSistema(usuarioSistema);
	}
	
	public void cargarMenu() {
		Log.debug("Inicializa revisión de alarmas cada " + SEGUNDOSPOLLING + " segundos");
		ControladorAlarmas.getInstance().startNotificadorAlarmas(SEGUNDOSPOLLING);
		Log.debug("GUI - dejar sólo la cabecera");
		dejarSoloLaCabecera();
		
		//menuWidget = new MenuPautasOAlarmasWidget();
		menuWidget = new PrincipalAlarmasWidget();
		mainVerticalRootPanel.add(menuWidget);
		
		//Inicializa lista de pacientes
		backendService.getPacientes(Controlador.getInstance().getUsuarioSistema(), new AsyncCallback<List<PacienteBean>>() {
			public void onFailure(Throwable caught) {
				Log.error("Error al solicitar los pacientes." + caught);
			}
			public void onSuccess(List<PacienteBean> pacientesBean) {
				setPacientes(pacientesBean);
			}
		});		
	}

	public void actualizarWidgetAlarmas() {
		Log.debug("Solicitada actualización de la lista de alarmas");
		if (menuWidget != null) {
			Log.debug("Actualizando widget de la lista de alarmas");
			menuWidget.actualizarListaAlarmas();
		}
	}

	public PacienteBean getPacienteBean(String id) {
		Log.debug("Obtener el bean del paciente " + id);
		for (PacienteBean pb : pacientes) {
			if (pb.getId().equalsIgnoreCase(id))
				return pb;
		}
		Log.error("Bean del paciente " + id + " no encontrado");
		return null;
	}
}
