package eu.atosresearch.health.prescripciones.client.widgets.alarmas;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.DialogBox;

import eu.atosresearch.health.prescripciones.client.BackendService;
import eu.atosresearch.health.prescripciones.client.BackendServiceAsync;
import eu.atosresearch.health.prescripciones.shared.AlarmaBean;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.ClickEvent;

public class HistoricoAlarmasDialogWidget extends DialogBox {

	private final BackendServiceAsync backendService = GWT.create(BackendService.class);
	private List<AlarmaBean> alarmasDelHistorico; 
	private ScrollPanel scrollPanel;
	
	public HistoricoAlarmasDialogWidget() {
		setHTML("Alarmas registradas");
		
		VerticalPanel verticalPanel = new VerticalPanel();
		setWidget(verticalPanel);
		verticalPanel.setSize("100%", "100%");
		
		Label lblHistricoDeAlarmas = new Label("Histórico de alarmas");
		lblHistricoDeAlarmas.setStyleName("labelTituloSeccion");
		verticalPanel.add(lblHistricoDeAlarmas);
		
		scrollPanel = new ScrollPanel();
		scrollPanel.setSize("910px", "300px");
		verticalPanel.add(scrollPanel);
		
		Button btnAceptar = new Button("Aceptar");
		btnAceptar.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Log.debug("Cerrar histórico");
				cerrarDialogo();
			}
		});
		btnAceptar.setStyleName("botonSeparado");
		verticalPanel.add(btnAceptar);
		
		Log.debug("Obtener y dibujar el grid con el histórico de alarmas");
		obtenerHistoricoAlarmas();
	}

	
	private void obtenerHistoricoAlarmas() { 
		backendService.obtenerHistoricoAlarmas(new AsyncCallback<List<AlarmaBean>>() {
			public void onFailure(Throwable caught) {
				// Show the RPC error message to the user
				Log.error("Error al solicitar el histórico de alarmas." + caught);
			}
			public void onSuccess(List<AlarmaBean> alarmasBean) {
				Log.info("Histórico de alarmas recibido");
				try {
					Log.info("Total alarmas (" + alarmasBean.size() + ")");
					alarmasDelHistorico = alarmasBean;
					
					Grid grid = new Grid(alarmasBean.size() + 1, 7);
					scrollPanel.setWidget(grid);
					grid.setSize("100%", "100%");
					
					Label lblg2 = new Label("Alarma");
					lblg2.setWidth("90px");
					Label lblg3 = new Label("Paciente");
					lblg3.setWidth("90px");
					Label lblg4 = new Label("Fecha");
					lblg4.setWidth("120px");
					Label lblg5 = new Label("Prioridad");
					lblg5.setWidth("100px");
					Label lblg6 = new Label("Descripción");
					lblg6.setWidth("200px");
					Label lblg7 = new Label("Acción");
					lblg7.setWidth("180px");
					Label lblg8 = new Label("Fecha acción");
					lblg8.setWidth("120px");
					
					grid.setWidget(0, 0, lblg2);
					grid.setWidget(0, 1, lblg3);
					grid.setWidget(0, 2, lblg4);
					grid.setWidget(0, 3, lblg5);
					grid.setWidget(0, 4, lblg6);
					grid.setWidget(0, 5, lblg7);
					grid.setWidget(0, 6, lblg8);
					
					int contadorFilas = 1;
					for (AlarmaBean alarma : alarmasBean) {
						grid.setText(contadorFilas, 0, alarma.getId());
						grid.setText(contadorFilas, 1, alarma.getPacienteId());
						grid.setText(contadorFilas, 2, alarma.getFecha());
						grid.setText(contadorFilas, 3, alarma.getPrioridad());
						grid.setText(contadorFilas, 4, alarma.getDescripcion());
						grid.setText(contadorFilas, 5, alarma.getQueSeHizo());
						grid.setText(contadorFilas, 6, alarma.getCuandoSeHizo());
						contadorFilas++;
					}
							
					
					grid.setCellSpacing(3);
					grid.setStyleName("gridHistoricoAlarmas");
					//scrollPanel.add(grid);
					
				} catch (NullPointerException npex) {
					Log.error(npex.getMessage());
				}
			}
		});
	}

	
	private void cerrarDialogo() {
		Log.debug("Cerrando el histórico...");
		hide();
	}
	
}
