package eu.atosresearch.health.prescripciones.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.apache.log4j.Logger;

import eu.atosresearch.health.prescripciones.client.BackendService;
import eu.atosresearch.health.prescripciones.server.alarma.AlarmaManager;
import eu.atosresearch.health.prescripciones.server.alarma.HistoricoAlarma;
import eu.atosresearch.health.prescripciones.server.authorization.Acceso;
import eu.atosresearch.health.prescripciones.server.pacientes.ConsultaPaciente;
import eu.atosresearch.health.prescripciones.server.vademecum.ConsultaMedicamento;
import eu.atosresearch.health.prescripciones.shared.*;

import com.gargoylesoftware.htmlunit.html.Util;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class BackendServiceImpl extends RemoteServiceServlet implements BackendService {

	static Logger logger = Logger.getLogger("BackendServiceImpl");
	public final static String LOG4J_FILE = "log4j.properties";

	public BackendServiceImpl() {
		logger.info("BackendServiceImpl loading...");
		//AlarmaManager.getInstance().startAlarmPolling(30);
		logger.info("BackendServiceImpl loaded.");
	}
	
	@Override
	public boolean accesoPermitido(String usuario, String pass) throws IllegalArgumentException {
		logger.info("Solicita acceso: " + usuario);
		return Acceso.autorizacion(usuario, pass);
	}

	@Override
	public List<PautaBean> getPautas(String usuario) throws IllegalArgumentException {
		logger.info("Obtener pautas almacenadas...");
		ArrayList<PautaBean> pautas = new ArrayList<PautaBean>();
		
		logger.debug("Listar ficheros:");
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//File directorio = new File("xmls");
		File directorio = new File(pathBuilder);
		
		logger.debug("Ruta: " + directorio.getAbsolutePath());
		File[] contenido = directorio.listFiles();
		for(int i = 0; i < contenido.length; i++) {
		    logger.debug(contenido[i] + " --> Nombre del fichero: " + contenido[i].getName());
		    if(contenido[i].getName().startsWith("pauta") && contenido[i].getName().endsWith(".xml")) {
		    	logger.debug(contenido[i] + " --> Se trata de un fichero de pauta");
		    	InputStream is;
				try {
					is = new FileInputStream(contenido[i].getAbsoluteFile());
					JAXBContext jc;
					jc = JAXBContext.newInstance(Pauta.class);
					Unmarshaller u = jc.createUnmarshaller();
					Pauta pauta = new ObjectFactory().createPauta();
					//Pauta 
					logger.debug("Unmarshalling Pauta desde el fichero...");
					pauta = (Pauta) u.unmarshal(is);
					logger.debug("Unmarshalling Pauta OK");
					PautaBean pautaBean = new PautaBean();
					pautaBean.setNombreFichero(contenido[i].getName());
					pautaBean.setFechaInicio(pauta.getFechaInicio());
					pautaBean.setHoraInicio(pauta.getHoraInicio());
					pautaBean.setPacienteId(pauta.getPacienteId());
					pautaBean.setId(pauta.getId());
					
					Dias dias = pauta.getDias();
					DiasBean diasBean = new DiasBean();
					
					for (Dia dia : dias.getDia()) {
						DiaBean diaBean = new DiaBean();
						diaBean.setId(dia.getId());
						
						TomasBean tomasBean = new TomasBean();
						for (Toma toma : dia.getTomas().getToma()) {
							TomaBean tomaBean = new TomaBean();
							tomaBean.setId(toma.getId());
							tomaBean.setHora(toma.getHora());

							MedicamentosBean medicamentosBean = new MedicamentosBean();
							for (Medicamento medicamento : toma.getMedicamentos().getMedicamento()) {
								MedicamentoBean medicamentoBean = new MedicamentoBean();
								medicamentoBean.setId(medicamento.getId());
								medicamentoBean.setPpoActivo(medicamento.getPpoActivo());
								medicamentoBean.setUbicacion(medicamento.getUbicacion());
								medicamentoBean.setObservaciones(medicamento.getObservaciones());
								medicamentoBean.setCriticidad(medicamento.getCriticidad());
								medicamentoBean.setDosis(medicamento.getDosis());
								
								medicamentosBean.getMedicamentoBean().add(medicamentoBean);
							}
							tomaBean.setMedicamentosBean(medicamentosBean);
							tomasBean.getTomaBean().add(tomaBean);
						}
						diaBean.setTomasBean(tomasBean);
						diasBean.getDiaBean().add(diaBean);
					}
					
					pautaBean.setDiasBean(diasBean);
					
					try {
						//Estado de la pauta (Latente - Activa - Inactiva)
						if (pautaBean.getNombreFichero().endsWith("_latente.xml")) {
							logger.debug("La pauta " + pauta.getId() + " está latente.");
							pautaBean.setEstado(PautaBean.ESTADOLATENTE);
						} else {
							List<TomaBean> tomas = pautaBean.obtenerTomas();
							if (Utilidades.convertirFecha(tomas.get(tomas.size()-1).getHora()).before(new Date()) ) {
								logger.debug("La pauta " + pauta.getId() + " está inactiva (ya terminó).");
								pautaBean.setEstado(PautaBean.ESTADOINACTIVA);
							} else {
								logger.debug("La pauta " + pauta.getId() + " está activa.");
								pautaBean.setEstado(PautaBean.ESTADOACTIVA);
							}							
						}
					}catch (Exception pex) {
						logger.error("Parse exception en la fecha de las tomas/pautas, pauta sin estado", pex);
					}
					
					
					//PautaBean terminada
					
					//Depende del usuario cuales se obtienen
					if (ConsultaPaciente.isTeleasistente(pautaBean.getPacienteId(), usuario)) {
						logger.debug("La pauta " + pautaBean.getId() + " sí puede ser accedida por " + usuario);
						pautas.add(pautaBean);
					} else {
						logger.debug("La pauta " + pautaBean.getId() + " no puede ser accedida por " + usuario);
					}
					
					logger.info("PAUTA <<" + contenido[i].getName() + ">> leída del almacenamiento");
				} catch (FileNotFoundException e) {
					logger.error(e);
					//e.printStackTrace();
				} catch (JAXBException e) {
					logger.error(e);
					e.printStackTrace();
				}
		    }
		}
		logger.debug("Todas las pautas obtenidas");
		return pautas;
	}
	
	@Override
	public List<TomaBean> getTomas(String usuario) throws IllegalArgumentException {
		logger.info("Obtener tomas almacenadas...");
		ArrayList<TomaBean> tomas = new ArrayList<TomaBean>();
		
		//To Do -> que dependa del usuario cuales se obtienen
		
		logger.debug("Listar ficheros:");

		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//File directorio = new File("xmls");
		File directorio = new File(pathBuilder);
		
		logger.debug("Ruta: " + directorio.getAbsolutePath());
		File[] contenido = directorio.listFiles();
		for(int i = 0; i < contenido.length; i++) {
		    logger.debug(contenido[i] + " --> Nombre del fichero: " + contenido[i].getName());
		    if(contenido[i].getName().startsWith("toma") && contenido[i].getName().endsWith(".xml")) {
		    	logger.debug(contenido[i] + " --> Se trata de un fichero de toma");
		    	InputStream is;
				try {
					is = new FileInputStream(contenido[i].getAbsoluteFile());
					JAXBContext jc;
					jc = JAXBContext.newInstance(Pauta.class);
					Unmarshaller u = jc.createUnmarshaller();
					Toma toma = new ObjectFactory().createToma();
					//Toma 
					logger.debug("Unmarshalling Toma desde el fichero...");
					toma = (Toma) u.unmarshal(is);
					logger.debug("Unmarshalling Toma OK");
	
					TomaBean tomaBean = new TomaBean();
					tomaBean.setId(toma.getId());
					tomaBean.setHora(toma.getHora());

					MedicamentosBean medicamentosBean = new MedicamentosBean();
					for (Medicamento medicamento : toma.getMedicamentos().getMedicamento()) {
						MedicamentoBean medicamentoBean = new MedicamentoBean();
						medicamentoBean.setId(medicamento.getId());
						medicamentoBean.setPpoActivo(medicamento.getPpoActivo());
						medicamentoBean.setUbicacion(medicamento.getUbicacion());
						medicamentoBean.setObservaciones(medicamento.getObservaciones());
						medicamentoBean.setCriticidad(medicamento.getCriticidad());
						medicamentoBean.setDosis(medicamento.getDosis());
						
						medicamentosBean.getMedicamentoBean().add(medicamentoBean);
					}
					tomaBean.setMedicamentosBean(medicamentosBean);
					
					//TomaBean terminada
					tomas.add(tomaBean);
					logger.info("TOMA <<" + contenido[i].getName() + ">> leída del almacenamiento");
				} catch (FileNotFoundException e) {
					logger.error(e);
					//e.printStackTrace();
				} catch (JAXBException e) {
					logger.error(e);
					e.printStackTrace();
				}
		    }
		}
		logger.debug("Todas las tomas obtenidas");
		return tomas;
	}

	@Override
	public List<PacienteBean> getPacientes(String usuario) throws IllegalArgumentException {
		logger.info("Listado de pacientes disponibles para: " + usuario);
		
		//Depende del usuario cuales se obtienen
		
		return ConsultaPaciente.pacientes(usuario);
	}

	//@Override
	//public List<PacienteBean> getPacientes(String usuario, String doctor) throws IllegalArgumentException {
	//	return ConsultaPaciente.pacientes(doctor);
	//}

	@Override
	public PacienteBean datosPaciente(String id) throws IllegalArgumentException {
		logger.info("Datos paciente: " + id);
		
		return ConsultaPaciente.datosPaciente(id);
	}

	@Override
	public MedicamentoBean getMedicamento(String idMedicamento) throws IllegalArgumentException {
		logger.info("GetMedicamento: " + idMedicamento);
		
		return ConsultaMedicamento.getMedicamento(idMedicamento);
	}

	@Override
	public List<AlarmaBean> getAlarmas(String usuario) throws IllegalArgumentException {
		logger.info("Obtener alarmas almacenadas... (usuario " + usuario + ")");
		ArrayList<AlarmaBean> alarmas = new ArrayList<AlarmaBean>();
		
		//Crear alarmas de tomas
		AlarmaManager.crearAlarmasTomas(getPautas(usuario), getTomas(usuario));
				
		logger.debug("Listar ficheros:");

		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//File directorio = new File("xmls");
		File directorio = new File(pathBuilder);
		
		logger.debug("Ruta: " + directorio.getAbsolutePath());
		File[] contenido = directorio.listFiles();
		for(int i = 0; i < contenido.length; i++) {
		    logger.debug(contenido[i] + " --> Nombre del fichero: " + contenido[i].getName());
		    if(contenido[i].getName().startsWith("alarma") && contenido[i].getName().endsWith(".xml")) {
		    	logger.debug(contenido[i] + " --> Se trata de un fichero de alarma");
		    	InputStream is;
				try {
					is = new FileInputStream(contenido[i].getAbsoluteFile());
					JAXBContext jc;
					jc = JAXBContext.newInstance(Alarma.class);
					Unmarshaller u = jc.createUnmarshaller();
					Alarma alarma = new Alarma();
					//Toma 
					logger.debug("Unmarshalling Alarma desde el fichero...");
					alarma = (Alarma) u.unmarshal(is);
					logger.debug("Unmarshalling Alarma OK");
	
					AlarmaBean alarmaBean = new AlarmaBean();
					alarmaBean.setId(alarma.getId());
					alarmaBean.setPacienteId(alarma.getPacienteId());
					alarmaBean.setFecha(alarma.getFecha());
					alarmaBean.setPrioridad(alarma.getPrioridad());
					alarmaBean.setDescripcion(alarma.getDescripcion());
					
					//AlarmaBean terminada
					
					//Depende del usuario cuales se obtienen
					if (ConsultaPaciente.isTeleasistente(alarmaBean.getPacienteId(), usuario)) {
						logger.debug("La alarma " + alarmaBean.getId() + " sí puede ser accedida por " + usuario);
						alarmas.add(alarmaBean);
					} else {
						logger.debug("La alarma " + alarmaBean.getId() + " no puede ser accedida por " + usuario);
					}
					
					logger.info("ALARMA <<" + contenido[i].getName() + ">> leída del almacenamiento");
				} catch (FileNotFoundException e) {
					logger.error(e);
					//e.printStackTrace();
				} catch (JAXBException e) {
					logger.error(e);
					e.printStackTrace();
				}
		    }
		}
		logger.debug("Todas las alarmas obtenidas");
		return alarmas;
	}

	@Override
	public boolean deleteAlarma(String alarmaId) throws IllegalArgumentException {
		logger.debug("Moviend alarma " + alarmaId + " a alarmas old.");
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//File alarmFile = new File("xmls/alarma" + alarmaId + ".xml");
		File alarmFile = new File(pathBuilder + "alarma" + alarmaId + ".xml");
	 
		//if(alarmFile.renameTo(new File("xmls/alarma" + alarmaId + ".xml"))) {
		if(alarmFile.renameTo(new File(pathBuilder + "old/alarma" + alarmaId + ".xml"))) {
			logger.debug("File " + "xmls/alarma" + alarmaId + ".xml" + " is moved successfully to " + "xmls/old/alarma" + alarmaId + ".xml");
			return true;
		} else {
			logger.error("File is failed to move from " + "xmls/alarma" + alarmaId + ".xml" + " to " + "xmls/old/alarma" + alarmaId + ".xml");
			return false;
		}
	}

	@Override
	public AlarmaBean almacenarAlarma(AlarmaBean alarma, String queSeHace, String cuandoSeHace) {
		logger.info("Almacenar alarma... " + alarma.getId());
		HistoricoAlarma.almacenarAlarma(alarma, queSeHace, cuandoSeHace);
		return alarma;
	}

	@Override
	public List<AlarmaBean> obtenerHistoricoAlarmas() {
		logger.info("Obtener histórico de alarmas");
		return HistoricoAlarma.obtenerHistorico();
	}

}
