package eu.atosresearch.health.prescripciones.server;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

import javax.xml.bind.JAXBException;
import javax.xml.namespace.NamespaceContext;
import javax.xml.namespace.QName;
import javax.xml.stream.Location;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.log4j.Logger;


import eu.atosresearch.health.prescripciones.shared.*;


@Path("/pauta")
public class PautaReceiver {
	
	static Logger logger = Logger.getLogger("PautaReceiver");
	
	
	@POST
	@Path("/nueva")
	@Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN})
	@Consumes(MediaType.APPLICATION_XML)
	public String nueva(Pauta pautaRecibida) throws JAXBException, IOException {
		logger.info("Pauta recibida");
		logger.debug(pautaRecibida);
		
		System.out.println("----------------------   PAUTA RECIBIDA   -----------------------");
		Pauta pauta = pautaRecibida;
		System.out.println("Pauta id:" + pauta.getId());
		System.out.println("Patient id:" + pauta.getPacienteId());
		System.out.println("Fecha inicio: " + pauta.getFechaInicio());
		System.out.println("Hora inicio: " + pauta.getHoraInicio());
		System.out.println("Días:");
		Dias dias = pauta.getDias();
		for (Dia dia : dias.getDia()) {
			System.out.println("   dia id=" + dia.getId());
			System.out.println("      tomas");
			for (Toma toma : dia.getTomas().getToma()) {
				System.out.println("         toma id=" + toma.getId());
				System.out.println("            hora: " + toma.getHora());
				System.out.println("            medicamentos");
				for (Medicamento medicamento : toma.getMedicamentos().getMedicamento()) {
					System.out.println("               medicamento id=" + medicamento.getId());
					System.out.println("                  principio activo:" + medicamento.getPpoActivo());
					System.out.println("                  ubicacion:" + medicamento.getUbicacion());
					System.out.println("                  observaciones:" + medicamento.getObservaciones());
					System.out.println("                  criticidad:" + medicamento.getCriticidad());
					System.out.println("                  dosis:" + medicamento.getDosis());
				}
			}
		}
		System.out.println("********   FIN PAUTA RECIBIDA   ***********");
		
		logger.debug("Almacenando pauta");
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//String nombreFichero = generarNombreUnico("xmls/pauta", "_latente.xml");
		//String nombreFichero = "xmls/pauta" + pauta.getId() + "_latente.xml";
		String nombreFichero = pathBuilder + "pauta" + pauta.getId() + "_latente.xml";
		File file = new File(nombreFichero);
	
		logger.debug("Creado fichero " + nombreFichero + " en " + file.getAbsoluteFile());
		FileOutputStream fOut = new FileOutputStream(file);
		logger.debug("Creado output stream");
			
		JAXBContext context = JAXBContext.newInstance(Pauta.class);
		Marshaller m = context.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.marshal(pautaRecibida, fOut);
		logger.debug("Fichero escrito correctamente");
		fOut.flush();
		fOut.close();
		logger.debug("Fichero cerrado correctamente");
		
		return "OK";
	}
	
	
	@POST
	@Path("/toma")
	@Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN})
	@Consumes(MediaType.APPLICATION_XML)
	public String toma(Toma tomaRecibida) throws JAXBException, IOException {
		logger.info("Toma recibida");
		logger.debug(tomaRecibida);
		
		System.out.println("----------------------   TOMA RECIBIDA   -----------------------");
		Toma toma = tomaRecibida;
		System.out.println("         toma id=" + toma.getId());
		System.out.println("            hora: " + toma.getHora());
		System.out.println("            medicamentos");
		for (Medicamento medicamento : toma.getMedicamentos().getMedicamento()) {
			System.out.println("               medicamento id=" + medicamento.getId());
			System.out.println("                  principio activo:" + medicamento.getPpoActivo());
			System.out.println("                  ubicacion:" + medicamento.getUbicacion());
			System.out.println("                  observaciones:" + medicamento.getObservaciones());
			System.out.println("                  criticidad:" + medicamento.getCriticidad());
			System.out.println("                  dosis:" + medicamento.getDosis());
		}
		System.out.println("********   TOMA RECIBIDA   ***********");
		
		logger.debug("Almacenando toma");
				
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//String nombreFichero = generarNombreUnico("xmls/toma", ".xml");
		//String nombreFichero = "xmls/toma" + toma.getId() + ".xml";
		String nombreFichero = pathBuilder + "toma" + toma.getId() + ".xml";
		File file = new File(nombreFichero);
	
		logger.debug("Creado fichero " + nombreFichero + " en " + file.getAbsoluteFile());
		FileOutputStream fOut = new FileOutputStream(file);
		logger.debug("Creado output stream");
			
		JAXBContext context = JAXBContext.newInstance(Toma.class);
		Marshaller m = context.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.marshal(tomaRecibida, fOut);
		logger.debug("Fichero escrito correctamente");
		fOut.flush();
		fOut.close();
		logger.debug("Fichero cerrado correctamente");
		
		return "OK";
	}
	
		
	@POST
	@Path("/alarma")
	@Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN})
	@Consumes(MediaType.APPLICATION_XML)
	public String alarma(Alarma alarmaRecibida) throws JAXBException, IOException {
		logger.info("Alarma recibida");
		logger.debug(alarmaRecibida);
		
		System.out.println("----------------------   ALARMA RECIBIDA   -----------------------");
		Alarma alarma = alarmaRecibida;
		System.out.println("         alarma id=" + alarma.getId());
		System.out.println("         patient id=" + alarma.getPacienteId());
		System.out.println("            fecha: " + alarma.getFecha());
		System.out.println("            prioridad: " + alarma.getPrioridad());
		System.out.println("            descripcion: " + alarma.getDescripcion());	
		System.out.println("********   ALARMA RECIBIDA   ***********");
		
		logger.debug("Almacenando alarma");
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//String nombreFichero = generarNombreUnico("xmls/alarma", ".xml");
		//String nombreFichero = "xmls/alarma" + alarma.getId() + ".xml";
		String nombreFichero = pathBuilder + "alarma" + alarma.getId() + ".xml";
		File file = new File(nombreFichero);
	
		logger.debug("Creado fichero " + nombreFichero + " en " + file.getAbsoluteFile());
		FileOutputStream fOut = new FileOutputStream(file);
		logger.debug("Creado output stream");
			
		JAXBContext context = JAXBContext.newInstance(Alarma.class);
		Marshaller m = context.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.marshal(alarmaRecibida, fOut);
		logger.debug("Fichero escrito correctamente");
		fOut.flush();
		fOut.close();
		logger.debug("Fichero cerrado correctamente");
		
		return "OK";
	}
	
		
	@POST
	@Path("/{id}/activar")
	@Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN})
	//@Consumes(MediaType.TEXT_PLAIN)
	public String activar(@PathParam("id") String pautaId) throws JAXBException, IOException {
		logger.info("Activar pauta " + pautaId);
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//File alarmFile = new File("xmls/pauta" + pautaId + "_latente.xml");
		File alarmFile = new File(pathBuilder + "pauta" + pautaId + "_latente.xml");
		 
		//if(alarmFile.renameTo(new File("xmls/pauta" + pautaId + ".xml"))) {
		if(alarmFile.renameTo(new File(pathBuilder + "pauta" + pautaId + ".xml"))) {
			logger.debug("File " + "xmls/pauta" + pautaId + "_latente.xml" + " is renamed successfully " + " with no _latente" + pautaId + ".xml");
			return "OK";
		} else {
			logger.error("File is failed to rename from " + "xmls/pauta" + pautaId + "_latente.xml" + " to " + "xmls/pauta" + pautaId + ".xml");
			return "KO";
		}
	}

	
	
	@GET
	@Path("/test")
	@Produces({MediaType.TEXT_PLAIN})
	//@Consumes(MediaType.TEXT_PLAIN)
	public String test() throws JAXBException, IOException {
		logger.info("TEST");
		return "test ok";
	}
	
	
	//Función para nombrar los ficheros de manera unívoca
	private String generarNombreUnico(String prefijo, String sufijo) {
		Date fecha = new Date();
		return prefijo + fecha.getTime() + sufijo;
	}


} 