package eu.atosresearch.health.prescripciones.server.alarma;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Timer;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.apache.log4j.Logger;

import eu.atosresearch.health.prescripciones.server.BackendServiceImpl;
import eu.atosresearch.health.prescripciones.server.Utilidades;
import eu.atosresearch.health.prescripciones.shared.Alarma;
import eu.atosresearch.health.prescripciones.shared.AlarmaBean;
import eu.atosresearch.health.prescripciones.shared.PautaBean;
import eu.atosresearch.health.prescripciones.shared.TomaBean;
//import eu.atosresearch.health.prescripciones.shared.Utilidades;


public class AlarmaManager {

	private static AlarmaManager INSTANCE = new AlarmaManager();

	public final static int MINUTOSMARGENALARMA = 30;

	static Logger logger = Logger.getLogger("AlarmaManager");
	
	private static Timer timer = new Timer(); 
	static List<AlarmaBean> alarmas;
	
	public static int contadorEstatico;
	
	//SINGLETON
	private AlarmaManager() {
	}
 
	public static AlarmaManager getInstance() {
        return INSTANCE;
    }    
	
	public static void startAlarmPolling(int segundos) {
		logger.info("Arranca polling de alarmas (" + segundos + " segundos intervalo)");
		alarmas = new ArrayList<AlarmaBean>(); 
		
		try {
			timer.schedule(new ComprobadorAlarma(), 0, segundos * 1000);
			AlarmaManager.getInstance().contadorEstatico ++;
		} catch (Exception e) {
			logger.error("Falló la inicialización del temporizador de alarmas", e);
			e.printStackTrace();
		}
		
	}

	
	public static void crearAlarmasTomas(List<PautaBean> pautas, List<TomaBean> tomasDelSistema) {
		logger.info("Creando alarmas de pautas y tomas no satisfechas");
		try {
			logger.debug("Recibidas " + pautas.size() + " pautas.");
			try {
				logger.debug("Recibidas " + tomasDelSistema.size() + " tomas.");
			} catch (NullPointerException npe) {
				logger.error("Tomas nulas", npe);
			}
			
			Date ahora = new Date();
			

			for (PautaBean pb : pautas) {
				try {
					if (pb.getEstado() == PautaBean.ESTADOACTIVA) {
						logger.debug("Pauta " + pb.getId() + " está activa, analizando posibles alarmas...");
						
						List<TomaBean> tomasPresentesEnLaPauta = pb.obtenerTomas();
						
						for (TomaBean tb : tomasPresentesEnLaPauta) {
							try {
								
								if (existeEstaToma(tomasDelSistema, tb.getId()))
									//logger.debug("Buscando alarma en toma " + tb.getId() + " - " + tb.getHora() + " existe y fue a las " + obtenerEstaToma(tomasDelSistema, tb.getId()).getHora() + ".");
									logger.debug("Buscando alarma en toma " + tb.getId() + " - " + tb.getHora() + " --> existe.");
								else
									logger.debug("Buscando alarma en toma " + tb.getId() + " - " + tb.getHora() + " no existe.");
								
								if (Utilidades.convertirFecha(tb.getHora()).before(ahora)) {
									logger.debug("Revisión de la toma descrita " + tb.getId() + ", que es antes que ahora, " + tb.getHora());
									//La toma existe
									if (existeEstaToma(tomasDelSistema, tb.getId())) {
										//if (Utilidades.tomaFueraDeTiempo(tb, obtenerEstaToma(tomasDelSistema, tb.getId()), MINUTOSMARGENALARMA)) {
										if (minutosRetrasoRespecto(obtenerEstaToma(tomasDelSistema, tb.getId()), tb) > MINUTOSMARGENALARMA) {
											logger.debug("Toma en estado de alarma " + tb.getId());
											
											if (alarmaYaExiste(tb.getId())) {
												logger.debug("Alarma " + tb.getId() + " ya existía.");
											} else {
												logger.debug("Alarma " + tb.getId() + " debe ser creada.");
												//Crear alarma, prioridad baja
												AlarmaBean alarmaNueva = new AlarmaBean();
												alarmaNueva.setId(tb.getId());
												alarmaNueva.setPacienteId(pb.getPacienteId());
												alarmaNueva.setFecha(Utilidades.convertirFecha(ahora));
												alarmaNueva.setPrioridad(AlarmaBean.PRIORIDADBAJA);
												alarmaNueva.setDescripcion("La toma " + tb.getId() + " se ha realizado fuera del tiempo recomendado: " + obtenerEstaToma(tomasDelSistema, tb.getId()).getHora() + " en lugar de " + tb.getHora());
												crearFicheroAlarma(alarmaNueva);
											}
										}
									} else {
									// La toma no existe
										
										//Si aún estamos en tiempo porque es muy reciente
										if (ahora.getTime() - Utilidades.convertirFecha(tb.getHora()).getTime() < (1000 * 60 * MINUTOSMARGENALARMA)) {
											logger.debug("La toma " + tb.getId() + " aún no se ha tomado pero aún está en marge.");
										} else {
											logger.debug("La toma " + tb.getId() + " no se ha tomado. Toma en estado de alarma.");
											if (alarmaYaExiste(tb.getId())) {
												logger.debug("Alarma " + tb.getId() + " ya existía.");
											} else {
												logger.debug("Alarma " + tb.getId() + " debe ser creada.");
												//Crear alarma, prioridad alta
												AlarmaBean alarmaNueva = new AlarmaBean();
												alarmaNueva.setId(tb.getId());
												alarmaNueva.setPacienteId(pb.getPacienteId());
												alarmaNueva.setFecha(Utilidades.convertirFecha(ahora));
												alarmaNueva.setPrioridad(AlarmaBean.PRIORIDADALTA);
												alarmaNueva.setDescripcion("La toma " + tb.getId() + " no se ha realizado");
												crearFicheroAlarma(alarmaNueva);
											}
										}
									}
								}
							} catch (Exception ex) {
								logger.error("Excepción al revisar tomas en las alarmas. " + "Toma " + tb.getId(), ex);
							}
						}
					} else {
						logger.debug("Pauta " + pb.getId() + " no está activa, no se analizan posibles alarmas.");
					}
				} catch (Exception ex) {
					logger.error("Excepción al revisar pautas en las alarmas " + "Pauta " + pb.getId(), ex);
				}
			}
			
		} catch (NullPointerException npe) {
			logger.error("Pautas nulas", npe);
		}
	}
	
	
	private static boolean crearFicheroAlarma(AlarmaBean alarmaRecibida) throws JAXBException, IOException {
		logger.info("Alarma a escribir en fichero");
		logger.debug(alarmaRecibida);
		
		System.out.println("----------------------   ALARMA A ESCRIBIR EN FICHERO   -----------------------");
		AlarmaBean alarma = alarmaRecibida;
		System.out.println("         alarma id=" + alarma.getId());
		System.out.println("         patient id=" + alarma.getPacienteId());
		System.out.println("            fecha: " + alarma.getFecha());
		System.out.println("            prioridad: " + alarma.getPrioridad());
		System.out.println("            descripcion: " + alarma.getDescripcion());	
		System.out.println("********   ALARMA A ESCRIBIR EN FICHERO   ***********");
		
		logger.debug("Almacenando alarma");
		
		Alarma alarmaXML = new Alarma();
		alarmaXML.setId(alarmaRecibida.getId());
		alarmaXML.setPacienteId(alarmaRecibida.getPacienteId());
		alarmaXML.setPrioridad(alarmaRecibida.getPrioridad());
		alarmaXML.setFecha(alarmaRecibida.getFecha());
		alarmaXML.setDescripcion(alarmaRecibida.getDescripcion());
		
		//String nombreFichero = generarNombreUnico("xmls/alarma", ".xml");
				
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//String nombreFichero = "xmls/alarma" + alarma.getId() + ".xml";
		String nombreFichero = pathBuilder + "alarma" + alarma.getId() + ".xml";
		
		File file = new File(nombreFichero);
	
		logger.debug("Creado fichero " + nombreFichero + " en " + file.getAbsoluteFile());
		FileOutputStream fOut = new FileOutputStream(file);
		logger.debug("Creado output stream");
			
		JAXBContext context = JAXBContext.newInstance(Alarma.class);
		Marshaller m = context.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		m.marshal(alarmaXML, fOut);
		logger.debug("Fichero escrito correctamente");
		fOut.flush();
		fOut.close();
		logger.debug("Fichero cerrado correctamente");
		
		return true;
	}
	
	private static boolean alarmaYaExiste(String alarmaId) {
		
		//En las alarmas y en el fichero Old
		
		String pathBuilder = BackendServiceImpl.class.getClassLoader().getResource(BackendServiceImpl.LOG4J_FILE).toString();
		pathBuilder = pathBuilder.substring(pathBuilder.indexOf("file:/") + 6, pathBuilder.indexOf("/WEB-INF/")) + "/xmls/";
		pathBuilder = pathBuilder.replace("%20", " ");
		logger.debug("Path for xmls: " + pathBuilder);
		
		//String nombreFichero = "xmls/alarma" + alarmaId + ".xml";
		//String nombreFicheroOld = "xmls/old/alarma" + alarmaId + ".xml";
		String nombreFichero = pathBuilder + "alarma" + alarmaId + ".xml";
		String nombreFicheroOld = pathBuilder + "old/alarma" + alarmaId + ".xml";
				
		//File file = new File(nombreFichero);
		File file = new File(nombreFichero);
		logger.debug("Comprobando fichero " + nombreFichero + " en " + file.getAbsoluteFile());
		//File fileOld = new File(nombreFicheroOld);
		File fileOld = new File(nombreFicheroOld);
		logger.debug("Comprobando fichero " + nombreFicheroOld + " en " + fileOld.getAbsoluteFile());
		
		if (file.exists() || fileOld.exists())
			return true;
		else
			return false;
	}
	
	private static boolean existeEstaToma(List<TomaBean> tomas, String tomaId) {
		for (TomaBean tb : tomas) {
			if (tb.getId().equalsIgnoreCase(tomaId))
				return true;
		}
		return false;
	}
	
	private static TomaBean obtenerEstaToma(List<TomaBean> tomas, String tomaId) {
		for (TomaBean tb : tomas) {
			if (tb.getId().equalsIgnoreCase(tomaId))
				return tb;
		}
		return null;
	}
	
	public static long minutosRetrasoRespecto(TomaBean tomaRecibida, Date fecha) throws ParseException {
		return (Utilidades.convertirFecha(tomaRecibida.getHora()).getTime() - fecha.getTime())  /  (1000 * 60) ;  
	}
	
	public static long minutosRetrasoRespecto(TomaBean tomaRecibida, TomaBean tomaPautada) throws ParseException {
		return (Utilidades.convertirFecha(tomaRecibida.getHora()).getTime() - Utilidades.convertirFecha(tomaPautada.getHora()).getTime())  /  (1000 * 60) ;  
	}
	
}
