package eu.atosresearch.health.prescripciones.server.alarma;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

import eu.atosresearch.health.prescripciones.server.BackendServiceImpl;
import eu.atosresearch.health.prescripciones.shared.AlarmaBean;

public class HistoricoAlarma {

	
	static Logger logger = Logger.getLogger(HistoricoAlarma.class);
	
	public static List<AlarmaBean> obtenerHistorico() {

		PropertiesConfiguration historicoAlarmas = new PropertiesConfiguration();
		try {
			historicoAlarmas.load(HistoricoAlarma.class.getClassLoader().getResourceAsStream("eu/atosresearch/health/prescripciones/server/alarma/historicoalarmas.properties"));
			logger.debug("Historico alarmas cargado correctamente.");
		} catch (ConfigurationException ex) {
		    logger.error("Exception cargando historico alarmas: ", ex);
		}
		
		List<AlarmaBean> alarmas = new ArrayList<AlarmaBean>();
		
		Iterator<String> it = historicoAlarmas.getKeys();
		while (it.hasNext()) {
			String[] datosAlarma = historicoAlarmas.getStringArray(it.next());
			AlarmaBean alarmaBean = new AlarmaBean();
			try {
				logger.debug("Histórico de alarmas: leída " + datosAlarma[0]);
				alarmaBean.setId(datosAlarma[0]);
				alarmaBean.setPacienteId(datosAlarma[1]);
				alarmaBean.setFecha(datosAlarma[2]);
				alarmaBean.setPrioridad(datosAlarma[3]);
				alarmaBean.setDescripcion(datosAlarma[4]);
				alarmaBean.setQueSeHizo(datosAlarma[5]);
				alarmaBean.setCuandoSeHizo(datosAlarma[6]);
			}catch (IndexOutOfBoundsException iobex) {
				logger.warn("La alarma " + datosAlarma[0] + " no tiene todos los campos", iobex);
			}
			alarmas.add(alarmaBean);
		}
		
		return alarmas;
	}
	
	
	public static boolean almacenarAlarma(AlarmaBean alarmaBean, String queSeHace, String cuandoSeHace) {
		PropertiesConfiguration historicoAlarmas = new PropertiesConfiguration();
		try {
			historicoAlarmas.load(HistoricoAlarma.class.getClassLoader().getResourceAsStream("eu/atosresearch/health/prescripciones/server/alarma/historicoalarmas.properties"));
			logger.debug("Historico alarmas cargado correctamente.");
		} catch (ConfigurationException ex) {
		    logger.error("Exception cargando historico alarmas: ", ex);
		}
		
		//#alarma=id,pacienteId,fecha,prioridad,descripcion,quesehizo,fechacuandosehizo
		historicoAlarmas.setProperty("alarma" + (new Date()).getTime(), 
				alarmaBean.getId()+","+alarmaBean.getPacienteId()+","+alarmaBean.getFecha()+","+alarmaBean.getPrioridad()+","+alarmaBean.getDescripcion()+","+queSeHace+","+cuandoSeHace);
		try {
			String pathBuilder = HistoricoAlarma.class.getClassLoader().getResource("eu/atosresearch/health/prescripciones/server/alarma/historicoalarmas.properties").getFile();
			pathBuilder = pathBuilder.replace("%20", " ");
			logger.debug("Histórico de alarmas que se prentende guardar: " + pathBuilder);
			
			historicoAlarmas.setFile(new File(pathBuilder));
			historicoAlarmas.save();
			logger.debug("Nueva entrada en el histórico de alarmas salvada: " + historicoAlarmas.getPath());
		} catch (ConfigurationException e) {
			logger.error("Error al cerrar el histórico en el sistema de ficheros", e);
			return false;
		}
		return true;
	}
	
}
